HALAKIM_IN_HOUR = 1080
HOURS_IN_DAY = 24
DAYS_IN_WEEK = 7
MONTHS_IN_YEAR = 12
MONTHS_IN_LEAP_YEAR = 13
YEARS_IN_CYCLE = 19
REGULAR_YEARS_IN_CYCLE = 12
LEAP_YEARS_IN_CYCLE =7

class MoladTime:
    weekday = None
    hour = None
    halakim = None
    
    def __init__(self, weekday, hour, halakim) -> None:
        self.halakim = halakim
        self.hour = hour
        self.weekday=weekday
        if self.halakim >= HALAKIM_IN_HOUR:
            self.hour+=self.halakim//HALAKIM_IN_HOUR
            self.halakim %= HALAKIM_IN_HOUR        
        if self.hour>=HOURS_IN_DAY:
            self.weekday+=self.hour//HOURS_IN_DAY
            self.hour%=HOURS_IN_DAY        
        if self.weekday>=DAYS_IN_WEEK:
            self.weekday%=DAYS_IN_WEEK

    def __add__(self,other) :
        
        halakim=self.halakim+other.halakim
        hour=self.hour+other.hour
        weekday=self.weekday+other.weekday
        if halakim >= HALAKIM_IN_HOUR:
            hour+=halakim//HALAKIM_IN_HOUR
            halakim =halakim%HALAKIM_IN_HOUR        
        if hour>=HOURS_IN_DAY:
            weekday+=hour//HOURS_IN_DAY
            hour%=HOURS_IN_DAY
        if weekday>DAYS_IN_WEEK:
            weekday%=DAYS_IN_WEEK       
        
        return MoladTime(weekday,hour,halakim)
    
    def __mul__(self,other):
        if not isinstance(other,int):
            raise TypeError       
              
        halakim = self.halakim*int(other)
        hour=self.hour*int(other)
        weekday=self.weekday*int(other)
        
        if halakim >= HALAKIM_IN_HOUR:
            hour+=(halakim//HALAKIM_IN_HOUR)
            halakim %= HALAKIM_IN_HOUR        
        if hour>=HOURS_IN_DAY:
            weekday+=(hour//HOURS_IN_DAY)
            hour%=HOURS_IN_DAY        
        if weekday>DAYS_IN_WEEK:
            weekday%=DAYS_IN_WEEK    
            
        return MoladTime(weekday,hour,halakim) 
    
    def __str__(self):
        return ("day: " + str(self.weekday +1) + " hour: " + str(self.hour) + " halakim: " + str(self.halakim))
    
 # מולד בהר"ד   
first_molad=MoladTime(1,5,204)
# כל חודש עודף על ימי השבוע ביום אחד שתים עשרה שעות ותשצג חלקים
sheerit_chodesh=MoladTime(1,12,793) 
sheerit_shana_pshuta=sheerit_chodesh*MONTHS_IN_YEAR
# print (sheerit_shana_pshuta)
sheerit_shana_meuberet=sheerit_chodesh*MONTHS_IN_LEAP_YEAR
# print (sheerit_shana_pshuta)
sheerit_machzor = (sheerit_shana_pshuta*REGULAR_YEARS_IN_CYCLE) + (sheerit_shana_meuberet*LEAP_YEARS_IN_CYCLE)


def get_molad_tishrei(year) ->MoladTime:
    molad=first_molad
    # לכל מחזור שלם מוסיפים שארית מחזור
    molad+=sheerit_machzor*(year//YEARS_IN_CYCLE)
    #למחזור הנוכחי מוסיפים לכל שנה פשוטה או מעוברת את השארית המתאימה
    for i in range(year%YEARS_IN_CYCLE):
        if isleapyear(i):
            molad+=sheerit_shana_meuberet
        else:
            molad+=sheerit_shana_pshuta
    return molad
            
def isleapyear(year):
    leap_years_in_cycle =[False, False, True, False, False, True, False, True, False, False, True, False, False, True, False, False, True, False, True]
    return leap_years_in_cycle[year%YEARS_IN_CYCLE]

#print(get_molad_tishrei(5782))

def get_rosh_hashana_day(year) -> int:
    molad = get_molad_tishrei(year)
    rosh_hashana_day = molad.weekday
    #דחיה ראשונה: מולד זקן
    if molad.hour >= 18:
        rosh_hashana_day += 1
    # דחייה שנייה: לא אדו ראש
    if rosh_hashana_day+1 == 1 or rosh_hashana_day+1 == 4 or rosh_hashana_day+1 == 6:
        rosh_hashana_day += 1    
    # דחייה שלישית: גטר"ד בשנה פשוטה
    if not isleapyear(year) \
        and molad.weekday+1 == 3 \
        and (molad.hour * HALAKIM_IN_HOUR + molad.halakim) >= (9 * HALAKIM_IN_HOUR + 204):
            rosh_hashana_day += 2
    #  דחייה רביעית: בטו תקפט בשנה שלאחר שנה מעוברת
    if isleapyear(year-1) \
        and molad.weekday+1 == 2 \
        and (molad.hour * HALAKIM_IN_HOUR + molad.halakim) >= (15 * HALAKIM_IN_HOUR + 589):
            rosh_hashana_day += 1            
    return rosh_hashana_day

def days_of_months(year) -> list:
    day1 = get_rosh_hashana_day(year)
    day2 = get_rosh_hashana_day(year+1)
    # ההפרש בין יום ראש השנה הזה לראש השנה הבא
    hefresh = day2 - day1 if day1 < day2 else day2 + DAYS_IN_WEEK - day1
    months = []
    if(not isleapyear(year)):
        if  hefresh == 3:
            months = [30,29,29,29,30,29,30,29,30,29,30,29]
        if hefresh == 4:
            months = [30,29,30,29,30,29,30,29,30,29,30,29]
        if hefresh == 5:
            months = [30,30,30,29,30,29,30,29,30,29,30,29]
    else:
        if hefresh == 5:
            months = [30,29,29,29,30,29,29,30,29,30,29,30,29]
        if hefresh == 6:
            months = [30,29,30,29,30,29,29,30,29,30,29,30,29]
        if hefresh == 0:
            months = [30,30,30,29,30,29,29,30,29,30,29,30,29]
    return months

def get_rosh_chodes_day(year, month) ->int:
    day=get_rosh_hashana_day(year)
    #מוסיף על ראש השנה לכל חודש את מספר הימים המתאים
    for i in range(0,len(days_of_months(year))):
        if i < month:
            day = (day + days_of_months(year)[i]) % DAYS_IN_WEEK
    return day
            
def get_weekday(year,month,day) ->int:
    weekday=get_rosh_chodes_day(year,month)
    weekday=(weekday+day)% DAYS_IN_WEEK
    return weekday

# כז חשוון תשפד אמור לצאת ביום שבת- יש להוסיף מספר אחד לכל הנתונים כי המניין מתחיל מ0
print(get_weekday(5783,1,26))

        
        
            
    
    
     
        
    



        
        
    